@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -109,6 +109,7 @@
 #include "utils/sha2.h"
 #include "auth/pool_passwd.h"
 #include "auth/scram.h"
+#include "auth/pool_auth.h"
 #include "utils/base64.h"
 #include "utils/elog.h"
 #include "utils/palloc.h"
@@ -1177,7 +1178,7 @@ build_server_final_message(scram_state *state)
 
 
 /*
- * Determinisitcally generate salt for mock authentication, using a SHA256
+ * Deterministically generate salt for mock authentication, using a SHA256
  * hash based on the username and a cluster-level secret key.  Returns a
  * pointer to a static buffer of size SCRAM_DEFAULT_SALT_LEN.
  */
