@@ -169,6 +169,8 @@ pool_start_query(POOL_QUERY_CONTEXT * query_context, char *query, int len, Node
 			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
 		query_context->skip_cache_commit = false;
+		query_context->atEnd = false;
+		query_context->partial_fetch = false;
 		session_context->query_context = query_context;
 		MemoryContextSwitchTo(old_context);
 	}
@@ -376,7 +378,16 @@ pool_virtual_main_db_node_id(void)
 	sc = pool_get_session_context(true);
 	if (!sc)
 	{
-		return REAL_MAIN_NODE_ID;
+		/*
+		 * We used to return REAL_MAIN_NODE_ID here.  Problem with it is, it
+		 * is possible that REAL_MAIN_NODE_ID could be changed
+		 * anytime. Suppose REAL_MAIN_NODE_ID == my_main_node_id == 1. Then
+		 * due to failback, REAL_MAIN_NODE_ID is changed to 0. Then
+		 * MAIN_CONNECTION(cp) will return NULL and any reference to it will
+		 * cause segmentation fault. To prevent the issue we should return
+		 * my_main_node_id instead.
+		 */
+		return my_main_node_id;
 	}
 
 	if (sc->in_progress && sc->query_context)
@@ -419,7 +430,7 @@ pool_virtual_main_db_node_id(void)
 	}
 
 	/*
-	 * No query context exists.  If in native replication mode, returns primary node
+	 * No query context exists.  If in streaming replication mode, returns primary node
 	 * if exists.  Otherwise returns my_main_node_id, which represents the
 	 * last REAL_MAIN_NODE_ID.
 	 */
@@ -665,7 +676,7 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		 * that any session object (i.e. named statement) does not remain in
 		 * the last session.
 		 */
-		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend: ", true) == 'E')
+		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend", true) == 'E')
 			reset_query_error = true;
 	}
 
@@ -877,7 +888,7 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			 * does not report what statement caused that error and make users
 			 * confused.
 			 */
-			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend: ", true);
+			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend", true);
 		}
 	}
 
