@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2023	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -142,8 +142,13 @@ typedef struct
 											 * used by parse_before_bind() */
 	bool		node_ids[MAX_NUM_BACKENDS];	/* backend node map which this message was sent to */
 	POOL_QUERY_CONTEXT *query_context;	/* query context */
+	/*
+	 * If "flush" message arrives, this flag is set to true until all buffered
+	 * message for frontend are sent out.
+	 */
+	bool		flush_pending;
 
-	bool		is_tx_started_by_multi_statement; /* true if an explicit transaction has been strated by
+	bool		is_tx_started_by_multi_statement; /* true if an explicit transaction has been started by
 													 multi statement query */
 }			POOL_PENDING_MESSAGE;
 
@@ -166,6 +171,7 @@ typedef enum
 	SI_SNAPSHOT_PREPARED
 } SI_STATE;
 
+
 /*
  * Per session context:
  */
@@ -305,10 +311,27 @@ typedef struct
 	/* Whether transaction is read only. Only used by Snapshot Isolation mode. */
 	SI_STATE	transaction_read_only;
 
+	/*
+	 * If true, the current message from backend must be flushed to frontend.
+	 * Set by read_kind_from_backend and reset by SimpleForwardToFrontend.
+	 */
+	bool		flush_pending;
+
 	bool		is_tx_started_by_multi_statement;	/* True if an explicit
 													 * transaction has been
 													 * started by a
 													 * multi-statement-query */
+	/*
+	 * True if query cache feature disabled until session ends.
+	 * This is set when SET ROLE/SET SESSION AUTHORIZATION executed.
+	 */
+	bool	query_cache_disabled;
+	/*
+	 * True if query cache feature disabled until current transaction ends.
+	 * This is set when REVOKE executed in a transaction.
+	 */
+	bool	query_cache_disabled_tx;
+
 }			POOL_SESSION_CONTEXT;
 
 extern void pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend);
@@ -375,6 +398,7 @@ extern void pool_check_pending_message_and_reply(POOL_MESSAGE_TYPE type, char ki
 extern POOL_PENDING_MESSAGE * pool_pending_message_find_lastest_by_query_context(POOL_QUERY_CONTEXT * qc);
 extern int	pool_pending_message_get_target_backend_id(POOL_PENDING_MESSAGE * msg);
 extern int	pool_pending_message_get_message_num_by_backend_id(int backend_id);
+extern void pool_pending_message_set_flush_request(void);
 extern void dump_pending_message(void);
 extern void pool_set_major_version(int major);
 extern void pool_set_minor_version(int minor);
@@ -396,15 +420,11 @@ extern bool is_tx_started_by_multi_statement_query(void);
 extern void set_tx_started_by_multi_statement_query(void);
 extern void unset_tx_started_by_multi_statement_query(void);
 
-#ifdef NOT_USED
-extern void pool_set_preferred_main_node_id(int node_id);
-extern int	pool_get_preferred_main_node_id(void);
-extern void pool_reset_preferred_main_node_id(void);
-#endif
+extern void set_query_cache_disabled(void);
+extern void unset_query_cache_disabled(void);
+extern bool query_cache_disabled(void);
+extern void set_query_cache_disabled_tx(void);
+extern void unset_query_cache_disabled_tx(void);
+extern bool query_cache_disabled_tx(void);
 
-#ifdef NOT_USED
-extern void pool_add_prep_where(char *name, bool *map);
-extern bool *pool_get_prep_where(char *name);
-extern void pool_delete_prep_where(char *name);
-#endif							/* NOT_USED */
 #endif							/* POOL_SESSION_CONTEXT_H */
