@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -18,7 +18,7 @@
  * is" without express or implied warranty.
  *
  *
- * pcp.h - master header file.
+ * pcp.h - pcp header file.
  */
 
 #ifndef PCP_H
@@ -38,9 +38,11 @@
 typedef struct PCPWDNodeInfo
 {
 	int			state;
+	int			membership_status;
 	char		nodeName[WD_MAX_HOST_NAMELEN];
 	char		hostName[WD_MAX_HOST_NAMELEN];	/* host name */
 	char		stateName[WD_MAX_HOST_NAMELEN]; /* state name */
+	char		membership_status_string[WD_MAX_HOST_NAMELEN]; /* membership status of this node */
 	int			wd_port;		/* watchdog port */
 	int			wd_priority;	/* node priority in leader election */
 	int			pgpool_port;	/* pgpool port */
@@ -51,11 +53,13 @@ typedef struct PCPWDNodeInfo
 typedef struct PCPWDClusterInfo
 {
 	int			remoteNodeCount;
+	int			memberRemoteNodeCount;
+	int			nodesRequiredForQuorum;
 	int			quorumStatus;
 	int			aliveNodeCount;
 	bool		escalated;
-	char		masterNodeName[WD_MAX_HOST_NAMELEN];
-	char		masterHostName[WD_MAX_HOST_NAMELEN];
+	char		leaderNodeName[WD_MAX_HOST_NAMELEN];
+	char		leaderHostName[WD_MAX_HOST_NAMELEN];
 	int			nodeCount;
 	PCPWDNodeInfo nodeList[1];
 }			PCPWDClusterInfo;
