@@ -29,6 +29,9 @@
 #include "pool.h"
 #include <sys/time.h>
 
+#define FORCE_QUERY_CACHE "/*FORCE QUERY CACHE*/"
+#define FORCE_QUERY_CACHE_COMMENT_SZ (sizeof(FORCE_QUERY_CACHE)-1)
+
 #define NO_QUERY_CACHE "/*NO QUERY CACHE*/"
 #define NO_QUERY_CACHE_COMMENT_SZ (sizeof(NO_QUERY_CACHE)-1)
 
@@ -250,7 +253,7 @@ extern uint32 hash_any(unsigned char *k, int keylen);
 
 extern POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION * frontend,
 												POOL_CONNECTION_POOL * backend,
-												char *contents, bool *foundp);
+												char *contents, bool use_fake_cache, bool *foundp);
 
 extern int pool_fetch_cache(POOL_CONNECTION_POOL * backend, const char *query, char **buf, size_t *len);
 extern int pool_catalog_commit_cache(POOL_CONNECTION_POOL * backend, char *query, char *data, size_t datalen);
@@ -277,7 +280,8 @@ extern POOL_QUERY_CACHE_ARRAY * pool_create_query_cache_array(void);
 extern void pool_discard_query_cache_array(POOL_QUERY_CACHE_ARRAY * cache_array);
 
 extern POOL_TEMP_QUERY_CACHE * pool_create_temp_query_cache(char *query);
-extern void pool_handle_query_cache(POOL_CONNECTION_POOL * backend, char *query, Node *node, char state);
+extern void pool_handle_query_cache(POOL_CONNECTION_POOL * backend, char *query, Node *node, char state,
+	bool partial_fetch);
 
 extern int	pool_init_memqcache_stats(void);
 extern POOL_QUERY_CACHE_STATS * pool_get_memqcache_stats(void);
@@ -302,6 +306,8 @@ extern void InvalidateQueryCache(int tableoid, int dboid);
 
 extern void pool_init_whole_cache_blocks(void);
 
-extern int delete_all_cache_on_memcached(void);
+extern void clear_query_cache(void);
+
+extern bool query_cache_delete_by_stmt(char *query, POOL_CONNECTION_POOL * backend);
 
 #endif							/* POOL_MEMQCACHE_H */
