@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -84,6 +84,7 @@ static void inform_watchdog_info(PCP_CONNECTION * frontend, char *buf);
 static void inform_node_info(PCP_CONNECTION * frontend, char *buf);
 static void inform_node_count(PCP_CONNECTION * frontend);
 static void process_reload_config(PCP_CONNECTION * frontend,char scope);
+static void process_log_rotate(PCP_CONNECTION * frontend,char scope);
 static void inform_health_check_stats(PCP_CONNECTION *frontend, char *buf);
 static void process_detach_node(PCP_CONNECTION * frontend, char *buf, char tos);
 static void process_attach_node(PCP_CONNECTION * frontend, char *buf);
@@ -92,6 +93,7 @@ static void process_status_request(PCP_CONNECTION * frontend);
 static void process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos);
 static void process_shutdown_request(PCP_CONNECTION * frontend, char mode, char tos);
 static void process_set_configuration_parameter(PCP_CONNECTION * frontend, char *buf, int len);
+static void process_invalidate_query_cache(PCP_CONNECTION * frontend);
 
 static void pcp_worker_will_go_down(int code, Datum arg);
 
@@ -267,9 +269,30 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			process_set_configuration_parameter(pcp_frontend, buf, buf_len);
 			break;
 
-		case 'L':				/* node count */
-			set_ps_display("PCP: processing node count request", false);
-			inform_node_count(pcp_frontend);
+		case 'B':				/* status request */
+			set_ps_display("PCP: processing status request request", false);
+			process_status_request(pcp_frontend);
+			break;
+
+		case 'C':				/* attach node */
+			set_ps_display("PCP: processing attach node request", false);
+			process_attach_node(pcp_frontend, buf);
+			break;
+
+		case 'D':				/* detach node */
+		case 'd':				/* detach node gracefully */
+			set_ps_display("PCP: processing detach node request", false);
+			process_detach_node(pcp_frontend, buf, tos);
+			break;
+
+		case 'F':
+			ereport(DEBUG1,
+					(errmsg("PCP processing request, stop online recovery")));
+			break;
+
+		case 'G':				/* invalidate query cache */
+			set_ps_display("PCP: processing invalidate query cache request", false);
+			process_invalidate_query_cache(pcp_frontend);
 			break;
 
 		case 'H':				/* health check stats */
@@ -282,46 +305,46 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			inform_node_info(pcp_frontend, buf);
 			break;
 
+		case 'J':				/* promote node */
+		case 'j':				/* promote node gracefully */
+			set_ps_display("PCP: processing promote node request", false);
+			process_promote_node(pcp_frontend, buf, tos);
+			break;
+
+		case 'L':				/* node count */
+			set_ps_display("PCP: processing node count request", false);
+			inform_node_count(pcp_frontend);
+			break;
+
 		case 'N':				/* process count */
 			set_ps_display("PCP: processing process count request", false);
 			inform_process_count(pcp_frontend);
 			break;
 
+		case 'O':				/* recovery request */
+			set_ps_display("PCP: processing recovery request", false);
+			process_recovery_request(pcp_frontend, buf);
+			break;
+
 		case 'P':				/* process info */
 			set_ps_display("PCP: processing process info request", false);
 			inform_process_info(pcp_frontend, buf);
 			break;
 
-		case 'W':				/* watchdog info */
-			set_ps_display("PCP: processing watchdog info request", false);
-			inform_watchdog_info(pcp_frontend, buf);
-			break;
-
-		case 'D':				/* detach node */
-		case 'd':				/* detach node gracefully */
-			set_ps_display("PCP: processing detach node request", false);
-			process_detach_node(pcp_frontend, buf, tos);
-			break;
-
-		case 'C':				/* attach node */
-			set_ps_display("PCP: processing attach node request", false);
-			process_attach_node(pcp_frontend, buf);
-			break;
-
 		case 'T':
 		case 't':
 			set_ps_display("PCP: processing shutdown request", false);
 			process_shutdown_request(pcp_frontend, buf[0], tos);
 			break;
 
-		case 'O':				/* recovery request */
-			set_ps_display("PCP: processing recovery request", false);
-			process_recovery_request(pcp_frontend, buf);
+		case 'V':				/* log rotate */
+			set_ps_display("PCP: processing log rotation request", false);
+			process_log_rotate(pcp_frontend, buf[0]);
 			break;
 
-		case 'B':				/* status request */
-			set_ps_display("PCP: processing status request request", false);
-			process_status_request(pcp_frontend);
+		case 'W':				/* watchdog info */
+			set_ps_display("PCP: processing watchdog info request", false);
+			inform_watchdog_info(pcp_frontend, buf);
 			break;
 
 		case 'Z':				/*reload config file */
@@ -329,17 +352,6 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			process_reload_config(pcp_frontend, buf[0]);
 			break;
 
-		case 'J':				/* promote node */
-		case 'j':				/* promote node gracefully */
-			set_ps_display("PCP: processing promote node request", false);
-			process_promote_node(pcp_frontend, buf, tos);
-			break;
-
-		case 'F':
-			ereport(DEBUG1,
-					(errmsg("PCP processing request, stop online recovery")));
-			break;
-
 		case 'X':				/* disconnect */
 			ereport(DEBUG1,
 					(errmsg("PCP processing request, client disconnecting"),
@@ -882,6 +894,7 @@ inform_node_info(PCP_CONNECTION * frontend, char *buf)
 			char		weight_str[20];
 			char		role_str[10];
 			char		standby_delay_str[20];
+			char		standby_delay_by_time_str[4];
 			char		status_changed_time_str[20];
 			char		code[] = "NodeInfo";
 			BackendInfo *bi = NULL;
@@ -918,6 +931,8 @@ inform_node_info(PCP_CONNECTION * frontend, char *buf)
 			}
 			snprintf(role_str, sizeof(role_str), "%d", role);
 
+			snprintf(standby_delay_by_time_str, sizeof(standby_delay_by_time_str), "%d", bi->standby_delay_by_time);
+
 			snprintf(standby_delay_str, sizeof(standby_delay_str), UINT64_FORMAT, bi->standby_delay);
 
 			snprintf(status_changed_time_str, sizeof(status_changed_time_str), UINT64_FORMAT, bi->status_changed_time);
@@ -932,6 +947,7 @@ inform_node_info(PCP_CONNECTION * frontend, char *buf)
 						  strlen(weight_str) + 1 +
 						  strlen(role_str) + 1 +
 						  strlen(nodes[i].pg_role) + 1 +
+						  strlen(standby_delay_by_time_str) + 1 +
 						  strlen(standby_delay_str) + 1 +
 						  strlen(bi->replication_state) + 1 +
 						  strlen(bi->replication_sync_state) + 1 +
@@ -947,6 +963,7 @@ inform_node_info(PCP_CONNECTION * frontend, char *buf)
 			pcp_write(frontend, weight_str, strlen(weight_str) + 1);
 			pcp_write(frontend, role_str, strlen(role_str) + 1);
 			pcp_write(frontend, nodes[i].pg_role, strlen(nodes[i].pg_role) + 1);
+			pcp_write(frontend, standby_delay_by_time_str, strlen(standby_delay_by_time_str) + 1);
 			pcp_write(frontend, standby_delay_str, strlen(standby_delay_str) + 1);
 			pcp_write(frontend, bi->replication_state, strlen(bi->replication_state) + 1);
 			pcp_write(frontend, bi->replication_sync_state, strlen(bi->replication_sync_state) + 1);
@@ -1069,6 +1086,7 @@ inform_node_count(PCP_CONNECTION * frontend)
 			(errmsg("PCP: informing node count"),
 			 errdetail("%d node(s) found", node_count)));
 }
+
 static void
 process_reload_config(PCP_CONNECTION * frontend, char scope)
 {
@@ -1100,6 +1118,32 @@ process_reload_config(PCP_CONNECTION * frontend, char scope)
 	do_pcp_flush(frontend);
 }
 
+static void
+process_log_rotate(PCP_CONNECTION * frontend, char scope)
+{
+	char            code[] = "CommandComplete";
+	int wsize;
+
+	if (scope == 'c' && pool_config->use_watchdog)
+	{
+		ereport(LOG,
+				(errmsg("PCP: sending command to watchdog to logrotate cluster")));
+
+		if (wd_execute_cluster_command(WD_COMMAND_LOGROTATE_CLUSTER, NULL) != COMMAND_OK)
+			ereport(ERROR,
+					(errmsg("PCP: error while processing log rotation request for cluster"),
+					 errdetail("failed to propagate log rotation command through watchdog")));
+	}
+
+	pool_signal_logrotate();
+
+	pcp_write(frontend, "v", 1);
+	wsize = htonl(sizeof(code) + sizeof(int));
+	pcp_write(frontend, &wsize, sizeof(int));
+	pcp_write(frontend, code, sizeof(code));
+	do_pcp_flush(frontend);
+}
+
 static void
 process_detach_node(PCP_CONNECTION * frontend, char *buf, char tos)
 {
@@ -1337,6 +1381,32 @@ process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos)
 	do_pcp_flush(frontend);
 }
 
+/*
+ * Process pcp_invalidate_query_cache
+ */
+static void
+process_invalidate_query_cache(PCP_CONNECTION * frontend)
+{
+	int			wsize;
+	char		code[] = "CommandComplete";
+
+	if (!pool_config->memory_cache_enabled)
+		ereport(ERROR,
+				(errmsg("query cache is not enabled")));
+
+	ereport(DEBUG1,
+			(errmsg("PCP: processing invalidate query cache")));
+
+	/* Set query cache invalidation request flag */
+	Req_info->query_cache_invalidate_request = true;
+
+	pcp_write(frontend, "g", 1);
+	wsize = htonl(sizeof(code) + sizeof(int));
+	pcp_write(frontend, &wsize, sizeof(int));
+	pcp_write(frontend, code, sizeof(code));
+	do_pcp_flush(frontend);
+}
+
 static void
 process_authentication(PCP_CONNECTION * frontend, char *buf, char *salt, int *random_salt)
 {
