@@ -22,7 +22,7 @@ export log=$dir/log
 fail=0
 ok=0
 timeout=0
-PGSOCKET_DIR=/tmp
+PGSOCKET_DIR=/tmp,/var/run/postgresql
 
 CRED=$(tput setaf 1)
 CGREEN=$(tput setaf 2)
@@ -220,6 +220,7 @@ do
 	check_segfault
 	if [ $? -eq 0 ];then
 		echo "fail: Segmentation fault detected" >> $log/$i
+		segfault=1
 		rtn=1
 	fi
 
@@ -230,7 +231,11 @@ do
 		echo "timeout."
 		timeout=`expr $timeout + 1`
 	else
-		echo ${CRED}"failed."${CNORM}
+		if [ "$segfault" = "1" ];then
+			echo ${CRED}"segfault."${CNORM}
+		else
+			echo ${CRED}"failed."${CNORM}
+		fi
 		fail=`expr $fail + 1`
 	fi
 
