@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2020	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -110,7 +110,7 @@ pool_search_relcache(POOL_RELCACHE * relcache, POOL_CONNECTION_POOL * backend, c
 	char	   *dbname;
 	int			i;
 	int			maxrefcnt = INT_MAX;
-	char		query[1024];
+	char		query[MAX_QUERY_LENGTH];
 	POOL_SELECT_RESULT *res = NULL;
 	int			index = 0;
 	int			local_session_id;
@@ -133,7 +133,7 @@ pool_search_relcache(POOL_RELCACHE * relcache, POOL_CONNECTION_POOL * backend, c
 
 	/*
 	 * Obtain database name and node id to be sent query.  If
-	 * relcache_query_target is RELQTARGET_LOADL_BALANCE_NODE, we consider
+	 * relcache_query_target is RELQTARGET_LOAD_BALANCE_NODE, we consider
 	 * load balance node id to be used to send queries.
 	 * 
 	 * Note that we need to use VALID_BACKEND_RAW, rather than VALID_BACKEND
@@ -229,7 +229,7 @@ pool_search_relcache(POOL_RELCACHE * relcache, POOL_CONNECTION_POOL * backend, c
 		if (!locked)
 		{
 			POOL_SETMASK2(&BlockSig, &oldmask);
-			pool_shmem_lock();
+			pool_shmem_lock(POOL_MEMQ_SHARED_LOCK);
 		}
 	    PG_TRY();
 		{
@@ -258,6 +258,19 @@ pool_search_relcache(POOL_RELCACHE * relcache, POOL_CONNECTION_POOL * backend, c
 	    if (pool_config->enable_shared_relcache)
 		{
 			query_cache_data = relation_cache_to_query_cache(res, &query_cache_len);
+			/*
+			 * So far, we have already obtained a lock. But to register
+			 * a query cache entry, we need to acquire an exclusive lock.
+			 * Unfortunately:
+			 * (1) we don't know if we already acquired an exclusive or not.
+			 * (2) we cannot escalate a shared lock to an exclusive lock.
+			 * So we release the lock and obtain an exclusive lock.  This may
+			 * create a window and thus we might try to register duplicate
+			 * query cache entry if other process is trying to register the same entry
+			 * in the window. But it should be harmless.
+			 */
+			pool_shmem_unlock();
+			pool_shmem_lock(POOL_MEMQ_EXCLUSIVE_LOCK);
 			pool_catalog_commit_cache(backend, query, query_cache_data, query_cache_len);
 		}
 	}
