@@ -203,11 +203,8 @@ wd_create_hb_send_socket(WdHbIf * hb_if)
 		{
 			if (geteuid() == 0) /* check root privileges */
 			{
-				struct ifreq i;
-
-				strlcpy(i.ifr_name, hb_if->if_name, sizeof(i.ifr_name));
-
-				if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, &i, sizeof(i)) == -1)
+				if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE,
+							   hb_if->if_name, strlen(hb_if->if_name)) == -1)
 				{
 					close(sock);
 					ereport(ERROR,
@@ -217,7 +214,7 @@ wd_create_hb_send_socket(WdHbIf * hb_if)
 				}
 				ereport(LOG,
 						(errmsg("creating socket for sending heartbeat"),
-						 errdetail("bind send socket to device: %s", i.ifr_name)));
+						 errdetail("bind send socket to device: %s", hb_if->if_name)));
 			}
 			else
 				ereport(LOG,
@@ -271,7 +268,7 @@ wd_create_hb_recv_socket(WdHbIf * hb_if)
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
 	hints.ai_protocol = 0;
-	hints.ai_flags = AI_NUMERICSERV;
+	hints.ai_flags = AI_NUMERICSERV | AI_PASSIVE;
 
 	if ((gai_ret = getaddrinfo(NULL, portstr, &hints, &res)) != 0)
 	{
@@ -336,11 +333,8 @@ wd_create_hb_recv_socket(WdHbIf * hb_if)
 			{
 				if (geteuid() == 0) /* check root privileges */
 				{
-					struct ifreq i;
-
-					strlcpy(i.ifr_name, hb_if->if_name, sizeof(i.ifr_name));
-
-					if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, &i, sizeof(i)) == -1)
+					if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE,
+								   hb_if->if_name, strlen(hb_if->if_name)) == -1)
 					{
 						ereport(LOG,
 								(errmsg("failed to create watchdog heartbeat receive socket"),
@@ -350,13 +344,14 @@ wd_create_hb_recv_socket(WdHbIf * hb_if)
 					}
 					ereport(LOG,
 							(errmsg("creating watchdog heartbeat receive socket."),
-							 errdetail("bind receive socket to device: \"%s\"", i.ifr_name)));
+							 errdetail("bind receive socket to device: \"%s\"", hb_if->if_name)));
 				}
 				else
 				{
 					ereport(LOG,
 							(errmsg("failed to create watchdog heartbeat receive socket."),
 							 errdetail("setsockopt(SO_BINDTODEVICE) requires root privilege")));
+					close(sock);
 					continue;
 				}
 			}
@@ -364,6 +359,7 @@ wd_create_hb_recv_socket(WdHbIf * hb_if)
 			ereport(LOG,
 					(errmsg("failed to create watchdog heartbeat receive socket"),
 					 errdetail("setsockopt(SO_BINDTODEVICE) is not available on this platform")));
+			close(sock);
 			continue;
 #endif
 		}
